'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getRangeId = getRangeId;
exports._buildDomainMapping = _buildDomainMapping;
exports._addProperties = _addProperties;
exports._addClasses = _addClasses;
exports._addBlankNodes = _addBlankNodes;
exports.generateFromGraph = generateFromGraph;

var _constants = require('./constants');

var constants = _interopRequireWildcard(_constants);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _addPrefix(uid) {
  return 'https://www.schesign.com/' + uid;
}

function getRangeId(context, property) {
  var uid = property.uid,
      range = property.range;

  if (range.type === 'LinkedClass') {
    return range.ref;
  }
  if (range.type === 'NestedObject') {
    return context.blankNodes[uid];
  }
  if (range.type === constants.BOOLEAN) {
    return constants.TYPE_BOOLEAN;
  }
  if (range.type === constants.ENUM) {
    return constants.TYPE_ENUM;
  }
  if (range.type === constants.TEXT) {
    if (range.format === constants.TEXT_FORMAT_URL) {
      return constants.TYPE_URL;
    }
    if (range.format === constants.TEXT_FORMAT_EMAIL) {
      return constants.TYPE_EMAIL;
    }
    if (range.format === constants.TEXT_FORMAT_HOSTNAME) {
      return constants.TYPE_HOSTNAME;
    }
    return constants.TYPE_TEXT;
  }
  if (range.type === constants.NUMBER) {
    if (range.format === constants.NUMBER_INT) {
      return constants.TYPE_INT;
    }
    if (range.format === constants.NUMBER_INT_8) {
      return constants.TYPE_INT_8;
    }
    if (range.format === constants.NUMBER_INT_16) {
      return constants.TYPE_INT_16;
    }
    if (range.format === constants.NUMBER_INT_32) {
      return constants.TYPE_INT_32;
    }
    if (range.format === constants.NUMBER_INT_64) {
      return constants.TYPE_INT_64;
    }
    if (range.format === constants.NUMBER_FLOAT_32) {
      return constants.TYPE_FLOAT_32;
    }
    if (range.format === constants.NUMBER_FLOAT_64) {
      return constants.TYPE_FLOAT_64;
    }
    return constants.TYPE_NUMBER;
  }
  if (range.type === constants.DATE) {
    if (range.format === constants.DATE_SHORT) {
      return constants.TYPE_SHORT_DATE;
    }
    if (range.format === constants.DATE_TIME) {
      return constants.TYPE_TIME;
    }
    return constants.TYPE_DATETIME;
  }

  throw new Error('Not expecting range: ' + JSON.stringify(range));
}

function _buildDomainMapping(context) {
  var domainMapping = {};
  var addRefs = function addRefs(parentId, propertySpecs) {
    propertySpecs.forEach(function (propertySpec) {
      if (!domainMapping[propertySpec.ref]) {
        domainMapping[propertySpec.ref] = [];
      }
      domainMapping[propertySpec.ref].push(parentId);
    });
  };
  Object.keys(context.classCache).forEach(function (key) {
    var classItem = context.classCache[key];
    addRefs(key, classItem.propertySpecs);
  });
  Object.keys(context.propertyCache).forEach(function (key) {
    var property = context.propertyCache[key];
    if (property.range.type === 'NestedObject') {
      addRefs(context.blankNodes[key], property.range.propertySpecs);
    }
  });
  return domainMapping;
}

function _addProperties(context) {
  var domainMapping = _buildDomainMapping(context);
  Object.keys(context.propertyCache).forEach(function (key) {
    var property = context.propertyCache[key];
    var node = {
      '@id': _addPrefix(key),
      '@type': 'rdf:Property',
      'rdf:label': property.label,
      'rdfs:range': { '@id': getRangeId(context, property) }
    };
    if (domainMapping[key]) {
      if (domainMapping[key].length === 1) {
        node['rdfs:domain'] = { '@id': _addPrefix(domainMapping[key][0]) };
      } else {
        node['rdfs:domain'] = domainMapping[key].map(function (id) {
          return { '@id': _addPrefix(id) };
        });
      }
    }
    context['@graph'].push(node);
  });
}

function _addClasses(context) {
  Object.keys(context.classCache).forEach(function (key) {
    var classItem = context.classCache[key];
    var node = {
      '@id': _addPrefix(key),
      '@type': 'rdfs:Class',
      'rdf:label': classItem.label
    };
    if (classItem.description) {
      node['rdfs:comment'] = classItem.description;
    }
    if (classItem.subClassOf) {
      node['rdfs:subClassOf'] = { '@id': classItem.subClassOf };
    }
    context['@graph'].push(node);
  });
}

function _addBlankNodes(context) {
  Object.keys(context.blankNodes).forEach(function (key) {
    context['@graph'].push({
      '@id': context.blankNodes[key],
      '@type': 'rdfs:Class'
    });
  });
}

function generateFromGraph(graph) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  /* Create a simple context obj to thread through */
  var context = {
    classCache: {},
    propertyCache: {},
    blankNodes: {},
    '@graph': []
  };

  /* Create a dict lookup for classes and properties for speed and convenience */
  var blankCounter = 0;
  graph.forEach(function (node) {
    if (node.type === 'Class') {
      context.classCache[node.uid] = node;
    } else if (node.type === 'Property') {
      context.propertyCache[node.uid] = node;
      if (node.range.type === 'NestedObject') {
        context.blankNodes[node.uid] = '_:a' + blankCounter;
        blankCounter++;
      }
    }
  });

  _addBlankNodes(context);
  _addClasses(context);
  _addProperties(context);

  return {
    '@context': {
      rdf: 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
      rdfs: 'http://www.w3.org/2000/01/rdf-schema#'
    },
    '@graph': context['@graph']
  };
}